@@ -1,5 +1,6 @@
 
 using System;
+using System.Linq;
 using System.Collections.Generic;
 
 using Mono.Unix;
@@ -19,6 +20,9 @@ namespace PdfMod.Actions
         public MoveAction (Document document, IEnumerable<Page> pages, int to_index) : base (document, pages)
         {
             this.to_index = to_index;
+            // Translators: {0} is the # of pages, {1} is a translated string summarizing the pages, eg "page 1"
+            Description = String.Format (Catalog.GetPluralString ("Move {1}", "Move {1}", Pages.Count),
+                Pages.Count, GlobalActions.GetPageSummary (Pages, 5));
         }
 
         public override void Undo ()
@@ -27,26 +31,18 @@ namespace PdfMod.Actions
                 throw new InvalidOperationException (Catalog.GetString ("Error trying to unmove pages"));
             }
 
-            for (int i = 0; i < old_indices.Length; i++) {
-                Console.WriteLine ("Trying to move page {0} back to index {1}", Pages[i], old_indices[i]);
-                Document.Move (old_indices[i], new Page [] { Pages[i] });
-            }
-
+            Document.Move (to_index, Pages.ToArray (), old_indices);
             old_indices = null;
         }
-        
+
         public override void Redo ()
         {
             if (old_indices != null) {
                 throw new InvalidOperationException (Catalog.GetString ("Error trying to move pages"));
             }
 
-            old_indices = new int[Pages.Count];
-            for (int i = 0; i < Pages.Count; i++) {
-                old_indices[i] = Document.IndexOf (Pages[i]);
-            }
-
-            Document.Move (to_index, Pages.ToArray ());
+            old_indices = Pages.Select<Page, int> (Document.IndexOf).ToArray ();
+            Document.Move (to_index, Pages.ToArray (), null);
         }
     }
 }
